<?php

declare(strict_types=1);

namespace Drupal\webform_maniterm\Plugin\WebformHandler;

use Drupal\Component\Serialization\Json;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\file\Entity\File;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Webform handler Maniterm API v2.
 *
 * @WebformHandler(
 *  id = "webform_maniterm",
 *  label = @Translation("Maniterm"),
 *  category = @Translation("External"),
 *  description = @Translation("Retrieves a caller ID from a prospect phone number."),
 *  cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_SINGLE,
 *  results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class WebformManitermHandler extends WebformHandlerBase {

  protected const API_URL = 'https://www.maniterm.com/api/json.php';

  protected const UPLOAD_DIR_BASE_PATH = 'public://webform_maniterm';

  /**
   * Guzzle HTTP client for requests.
   *
   * @var \GuzzleHttp\Client
   */
  protected Client $httpClient;

  /**
   * Logger channel for webform.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected LoggerChannelInterface $logger;

  /**
   * File system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    $instance->httpClient = $container->get('http_client');
    $instance->logger = $instance->getLogger('webform');
    $instance->fileSystem = $container->get('file_system');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'secret_token' => '',
      'prospect_phone_elt' => '',
      'prospect_test_phone_number' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    // The campaign specific secret token.
    $form['webform_maniterm']['secret_token'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('Secret token'),
      '#description' => $this->t('Maniterm campaign specific secret token.'),
      '#default_value' => $this->configuration['secret_token'] ?? '',
    ];

    // The prospect phone element.
    $webform_elements = $this->webform->getElementsDecodedAndFlattened();
    $tel_elements = array_filter($webform_elements, function ($value) {
      return $value['#type'] === 'tel';
    });
    array_walk($tel_elements, function (&$value, $key) {
      $value = $value['#title'];
    });
    $callerid_elements = $webform_elements;
    array_walk($callerid_elements, function (&$value, $key) {
      $value = $value['#title'];
    });

    $form['webform_maniterm']['prospect_phone_elt'] = [
      '#type' => 'select',
      '#options' => $tel_elements,
      '#required' => TRUE,
      '#title' => $this->t('Prospect phone element'),
      '#description' => $this->t('Webform element name for the prospect phone number.'),
      '#default_value' => $this->configuration['prospect_phone_elt'] ?? '',
    ];

    $form['webform_maniterm']['callerid_elt'] = [
      '#type' => 'select',
      '#options' => $callerid_elements,
      '#required' => FALSE,
      '#title' => $this->t('Caller ID element'),
      '#description' => $this->t('Webform element name for the caller ID request parameter.'),
      '#default_value' => $this->configuration['callerid_elt'] ?? '',
    ];

    // The prospect phone number for testing the Maniterm API.
    $form['webform_maniterm']['prospect_test_phone_number'] = [
      '#type' => 'tel',
      '#required' => TRUE,
      '#title' => $this->t('Prospect test phone number'),
      '#description' => $this->t('This number will be used to test the Maniterm API during the form validation.'),
      '#default_value' => $this->configuration['prospect_test_phone_number'] ?? '',
    ];

    $form['webform_maniterm']['callerid_test_value'] = [
      '#type' => 'textfield',
      '#required' => FALSE,
      '#title' => $this->t('Caller ID test value'),
      '#description' => $this->t('This ID will be used to test the Maniterm API during the form validation.'),
      '#default_value' => $this->configuration['callerid_test_value'] ?? '',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state): void {
    // Validates the Maniterm API response.
    try {
      $values = $form_state->getValue('webform_maniterm');
      $form_params = [
        'action' => 'lock_cli',
        'secret_token' => $values['secret_token'],
        'prospect_number' => str_replace(' ', '', $values['prospect_test_phone_number']),
      ];
      if (!empty($values['callerid_test_value'])) {
        $form_params['callerid'] = $values['callerid_test_value'];
      }
      $response = $this->httpClient->request('POST', self::API_URL, [
        'form_params' => $form_params,
      ]);

      // Validates the Maniterm API response has no error.
      $response_body = Json::decode($response->getBody()->getContents());
      $data = $response_body['data'] ?? [];
      if (empty($data)) {
        $form_state->setError(
          $form['settings']['webform_maniterm']['secret_token'],
          $this->t('Failed to get a valid Maniterm response.')
        );
      }
      elseif (!empty($data['error'])) {
        $form_state->setError(
          $form['settings']['webform_maniterm']['secret_token'],
          $this->t('Failed to get a valid Maniterm response with the following error: @error', [
            '@error' => $data['error'],
          ])
        );
      }
      else {
        $this->messenger()->addMessage(
          $this->t('The Maniterm API test successfully returned the following caller ID: @callerid.', [
            '@callerid' => $data['callerid'],
          ])
        );
      }
    }
    catch (GuzzleException $e) {
      $form_state->setError(
        $form['settings']['webform_maniterm']['secret_token'],
        $this->t('Failed to get a valid Maniterm response with the following error: @error', [
          '@error' => $data['error'],
        ])
      );
    }

    // Validates the add of the webform_maniterm element to the webform.
    try {
      $elements = $this->webform->getElementsDecoded();
      $webform_maniterm_elements = [
        'webform_maniterm' => [
          '#type' => 'custom_composite',
          '#title' => 'Manifone data',
          '#multiple' => FALSE,
          '#access' => FALSE,
          '#element' => [
            'callerid' => [
              '#type' => 'tel',
              '#title' => 'Caller ID',
            ],
            'vcf' => [
              '#type' => 'file',
              '#title' => '.vcf file',
            ],
            'company_name' => [
              '#type' => 'textfield',
              '#title' => 'Company name',
            ],
            'logo_url' => [
              '#type' => 'url',
              '#title' => 'Logo URL',
            ],
          ],
        ],
      ];
      $elements = array_merge($elements, $webform_maniterm_elements);
      $this->webform->setElements($elements);
      $this->webform->save();
    }
    catch (\Exception $e) {
      $form_state->setError(
        $form['webform_maniterm'],
        $this->t('Failed to add webform_maniterm element')
      );
    }

    // Validates the creation of the upload directory.
    $dir_path = self::UPLOAD_DIR_BASE_PATH . '/' . $this->webform->id();
    if (!$this->fileSystem->prepareDirectory($dir_path, FileSystemInterface::CREATE_DIRECTORY)) {
      $form_state->setError(
        $form['webform_maniterm'],
        $this->t('Failed to create a directory @dir_path', [
          '@dir_path' => $dir_path,
        ])
      );
    }

  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    // Saves config.
    $values = $form_state->getValue('webform_maniterm');
    $this->configuration['secret_token'] = $values['secret_token'];
    $this->configuration['prospect_phone_elt'] = $values['prospect_phone_elt'];
    $this->configuration['callerid_elt'] = $values['callerid_elt'];
    $this->configuration['prospect_test_phone_number'] = $values['prospect_test_phone_number'];
    $this->configuration['callerid_test_value'] = $values['callerid_test_value'];
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(WebformSubmissionInterface $webform_submission, $update = TRUE): void {
    // Sets the Maniterm API response to the webform_maniterm element data.
    $form_params = $this->buildRequestParams($webform_submission, $update);

    if (!$form_params) {
      return;
    }

    try {
      $response = $this->httpClient->request('POST', self::API_URL, [
        'form_params' => $form_params,
      ]);

      $response_body = Json::decode($response->getBody()->getContents());
      $data = $response_body['data'] ?? '';

      if (empty($data)) {
        $this->logger->error(
          $this->t('Failed to get a valid Maniterm response.')
        );
      }
      else {
        $dir_path = self::UPLOAD_DIR_BASE_PATH . '/' . $this->webform->id();
        $file_path = $this->fileSystem->saveData(
          $data['vcf'], $dir_path . '/' . $data['callerid'] . '.vcf',
          FileSystemInterface::EXISTS_RENAME
        );

        $files = \Drupal::entityTypeManager()
          ->getStorage('file')
          ->loadByProperties(['uri' => $file_path]);
        $file = reset($files);

        if (!$file) {
          $file = File::create(['uri' => $file_path]);
          $file->setOwnerId(1);
          $file->setPermanent();
          $file->save();
        }

        $data['vcf'] = $file->id();

        $webform_submission->setElementData('webform_maniterm', Json::encode($data));
      }
    }
    catch (GuzzleException $e) {
      $this->logger->error(
        $this->t('Failed to get a valid Maniterm response with the following error:: @error', [
          '@error' => $e->getMessage(),
        ])
      );
    }
  }

  /**
   * Builds the request parameters.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   The webform submission.
   * @param bool $update
   *   Whether to update the submission.
   *
   * @return array
   *   The request parameters.
   */
  public function buildRequestParams(WebformSubmissionInterface $webform_submission, $update = TRUE): array {
    $prospect_phone_elt = $this->configuration['prospect_phone_elt'];
    $prospect_phone = $webform_submission->getElementData($prospect_phone_elt);
    $form_params = [
      'action' => 'lock_cli',
      'secret_token' => $this->configuration['secret_token'],
      'prospect_number' => str_replace(' ', '', $prospect_phone),
    ];
    if (!empty($this->configuration['callerid_elt'])) {
      $callerid_elt = $this->configuration['callerid_elt'];
      $form_params['callerid'] = $webform_submission->getElementData($callerid_elt);
    }

    return $form_params;
  }

}
